package Network.Algorithm;

import Network.Representation.*;
import Network.Algorithm.*;

/**
 * This interface serves as a general template for classes that will represent algorithms.
 * The generic way to carry out an algorithm on a network is: first to create an instance of 
 * the desired algorithm class and call the <code>setTargetNetwork</code> of this class to 
 * direct the algorithm on a desired network and then use the algorithm's core method that 
 * carries out the steps of the algorithm. A default implementation for this interface is given in
 * <code>AlgorithmImpelementation</code>, the users are encouraged to subclass this class because
 * of tiny details that should be taken care of when setting up the algorithm information fields
 * while setting the target_network for an algorithm.
 * @see Network.Algorithm.AlgorithmImplementation
 */
 
public interface Algorithm {
    
    void setTargetNetwork(Network setNetwork);
    void setTargetNetwork(Network setNetwork,boolean removeOldAlgInfo);
    Network getTargetNetwork();
    int getAlgorithmOrder();
    NodeAlgInfo getInstanceOfNodeAlgInfo();
    ArcAlgInfo getInstanceOfArcAlgInfo();
    NodeAlgInfo getGeneralAlgNodeInfo(Node whichNodesInfo);
    ArcAlgInfo getGeneralAlgArcInfo(Arc whichArcsInfo);
    
}
