package Network.Algorithm.SimplexAlgorithm;


import Network.Representation.*;


/**
 * This class is only for use by <code>SimpexAlgorithm</code>. Users never have to create an instance of this class.
 */
final class SinglyLinkedCandidateList implements CandidateList {

    private SimplexAlgorithm ownerAlgorithm;
    private NodeList lastNodeTestedInList;
    private int listSize;
    private int maxListSize;
    private int maxNoOfGivenArcs;
    private int noOfGivenArcs;   
    private ArcList arcList;
    
    //Here!
    private static final ArcListFactory arcListFactory = new ArcListFactory(1000, 500);
    
    
    public SinglyLinkedCandidateList(int size, int cycleLength, Network network,SimplexAlgorithm owner){
        initialize(size, cycleLength, network, owner);
    }
    
    
   public final void initialize(int size, int cycleLength, Network network,SimplexAlgorithm owner){
        lastNodeTestedInList=network.getNodeListEnd();
        maxListSize=size;
        cleanList();
        maxNoOfGivenArcs=cycleLength;
        noOfGivenArcs=cycleLength;
        ownerAlgorithm=owner;
    }
    
    
    public final Arc bestCandidate(double optimalityTolerance, boolean isDoubleCosts) {
        Arc bestArcSoFar;                       /*bestarc*/
        double bestCostSoFar;                   /*bestcost*/
        Arc bestArcForCurrentNode;              /*bestnodearc*/
        double bestCostForCurrentNode;          /*bestnodecost*/
        Arc dummyArcCursor;                     /*tmparcp*/
        Arc currentArc;                         /*tmparc*/
        ArcList currentArcIterator; //replaces currentArc, tmparc*/
        double checkedRC;              /*checkcost*/
        Node nodeToPrice;                       /*nodetoprice*/
        NodeList nodeToPriceIterator;
        Node nodeCursor;                        /*whichnode*/
        NodeList nodeCursorIterator;
        Node stoppingNode;                      /*stopnode*/
        NodeList stoppingNodeIterator;
        double reducedCost;                     /*dualval*/
        int i;
        
        bestArcSoFar=null;
        bestCostSoFar=0;
        if (noOfGivenArcs < maxNoOfGivenArcs){
            currentArcIterator=arcList;
            for (i=0; i<listSize; i++){
                currentArc=currentArcIterator.elem;
                if (ownerAlgorithm.getAlgArcInfo(currentArc).flow==0){
                    checkedRC=  ownerAlgorithm.getAlgNodeInfo(currentArc.getTailNode()).dualVariable+currentArc.getCost()-ownerAlgorithm.getAlgNodeInfo(currentArc.getHeadNode()).dualVariable;
                }
                else if (ownerAlgorithm.getAlgArcInfo(currentArc).flow==currentArc.getUpperBound()){
                    checkedRC= -ownerAlgorithm.getAlgNodeInfo(currentArc.getTailNode()).dualVariable-currentArc.getCost()+ownerAlgorithm.getAlgNodeInfo(currentArc.getHeadNode()).dualVariable;
                }
                else{
                    checkedRC=  ownerAlgorithm.getAlgNodeInfo(currentArc.getTailNode()).dualVariable+currentArc.getCost()-ownerAlgorithm.getAlgNodeInfo(currentArc.getHeadNode()).dualVariable;
                    if (checkedRC>0) checkedRC=-checkedRC;
                }
                if (checkedRC<bestCostSoFar) {
                    bestCostSoFar=checkedRC;
                    bestArcSoFar=currentArc;
                }
                //NOTE TO HUSEYIN: Was the next line omitted by accident?
                //Otherwise I can't see how you ever scan an already-built list . . .
                //and nothing varies with the loop index i.  J.A.S., 8/29/98
                currentArcIterator=currentArcIterator.next;
            }
        }
        
        if ((isDoubleCosts) && (bestArcSoFar!=null) && (bestCostSoFar > (-optimalityTolerance * (1.0+Math.abs(bestArcSoFar.getCost())+Math.abs(ownerAlgorithm.getAlgNodeInfo(bestArcSoFar.getTailNode()).dualVariable)+Math.abs(ownerAlgorithm.getAlgNodeInfo(bestArcSoFar.getHeadNode()).dualVariable))))) {
            bestArcSoFar=null;
        }

                            
        if (bestArcSoFar==null){
            //NOTE TO HUSEYIN: Was the next line omitted by accident?
            //Otherwise you just keep adding to the same list and never
            //rebuilding it.  J.A.S., 8/29/98
            cleanList();
            stoppingNodeIterator=lastNodeTestedInList;
            nodeCursorIterator=stoppingNodeIterator;
            do {
                nodeCursorIterator=ownerAlgorithm.getTargetNetwork().getNextNodeInList(nodeCursorIterator);
                nodeToPriceIterator=nodeCursorIterator;
                bestCostForCurrentNode=0;
                bestArcForCurrentNode=null;
                currentArcIterator=nodeToPriceIterator.elem.getOutBoundArcs();
                reducedCost=ownerAlgorithm.getAlgNodeInfo( (nodeToPriceIterator.elem) ).dualVariable;
                while (currentArcIterator!=null) {
                    currentArc=currentArcIterator.elem;
                    if ((ownerAlgorithm.getAlgArcInfo(currentArc).flow)==0) {
                        checkedRC= reducedCost+currentArc.getCost()-ownerAlgorithm.getAlgNodeInfo(currentArc.getHeadNode()).dualVariable;
                    }
                    else if ((ownerAlgorithm.getAlgArcInfo(currentArc).flow)==currentArc.getUpperBound()) {
                        checkedRC=-reducedCost-currentArc.getCost()+ownerAlgorithm.getAlgNodeInfo(currentArc.getHeadNode()).dualVariable;
                    }
                    else {
                        checkedRC= reducedCost+currentArc.getCost()-ownerAlgorithm.getAlgNodeInfo(currentArc.getHeadNode()).dualVariable;
                        if (checkedRC>0) {checkedRC=-checkedRC;}
                    }
                    if (checkedRC<bestCostForCurrentNode){
                        bestCostForCurrentNode=checkedRC;
                        bestArcForCurrentNode=currentArc;
                    }
                    currentArcIterator=currentArcIterator.next;
                }
                if (bestArcForCurrentNode!=null) {
                    /**************************************************************************/
                    if (arcList != null)
                        arcList.addAfter(arcListFactory.make(bestArcForCurrentNode));
                    else
                        arcList = arcListFactory.make(bestArcForCurrentNode);
                    listSize++;
                    if (bestCostForCurrentNode<bestCostSoFar) {
                        bestCostSoFar=bestCostForCurrentNode;
                        bestArcSoFar=bestArcForCurrentNode;
                    }
                }
            }
            while ((listSize<maxListSize) && (!(nodeCursorIterator.equals(stoppingNodeIterator))));
            
            if ((isDoubleCosts==true) && (bestArcSoFar!=null) && (bestCostSoFar > (-optimalityTolerance * (1.0+Math.abs(bestArcSoFar.getCost())+Math.abs(ownerAlgorithm.getAlgNodeInfo(bestArcSoFar.getTailNode()).dualVariable)+Math.abs(ownerAlgorithm.getAlgNodeInfo(bestArcSoFar.getHeadNode()).dualVariable))))) {
                bestArcSoFar=null;
            }
            
            
            
            lastNodeTestedInList=nodeCursorIterator;
            noOfGivenArcs=0;
        }
        noOfGivenArcs++;
        return(bestArcSoFar);
    }
                    
                        
    public final void cleanList()
    {
        ArcList clean = arcList;
        while (clean != null) {
            ArcList next = clean.next;
            clean.elem = null; //Helps GC
            arcListFactory.unmake(clean);
            clean = next;
        }
        listSize = 0;
        arcList = null;
    }

}
