package Network.Algorithm.TreeAlgorithm;

import Network.Algorithm.*;
import Network.Representation.*;

public class TreeNodeInfo implements NodeAlgInfo {
    
    /**
     * The node that preceeds this node on the path that goes from the root node to this node.
     */
    public Node treePredecessor;
    
    /**
     * Pointer to a node such that if these pointers are followed you will end up with the node
     * you have started with and in the process you will go thru every node.
     * Also it is guaranteed that if you start at root node and follow the thread then
     * you will go thru the predecessor of a node before you go thru the node itself.
     */ 
    public Node reverseThread;
    
    /**
     * Pointer to a node that has the same characteristics as <code>reverseThread</code>
     * but this path is in the opposite direction.
     */
    public Node forwardThread;
    
    /**
     * The arc that link this node to the subtree structure that contains the root node.
     */
    public Arc predecessorLink;
    
    /** 
     * Represents whether this node is a tail node or a head node for <code>predecessorLink</code>.
     */
    public boolean downPointer;
    
    /**
     * Shows the number of arcs between this node and the root node on the path that follows the tree and
     * that goes from this node to root node.
     */
    public int depth;
    public int flag;
    
    public final Node getTreePredecessor() {
        return treePredecessor;
    }
    
    public final Node getReverseThread() {
        return reverseThread;
    }
    
    public final Node getForwardThread() {
        return forwardThread;
    }
    
    public final boolean isDown(){
        return downPointer;
    }
    
    public final int getDepth() {
        return depth;
    }
            
    
    public String toString() {
        return Integer.toString(depth)+" "+treePredecessor.toString()+" "+predecessorLink.toString();
    }
    
}
    