package Network.Representation;

import Network.Representation.*;

/** 
 * This interface provides a general template for network. The networks serve as masters on nodes and arcs: they
 * give the orders to nodes and arcs. You can add ordelete a node or an ac thru the networks relevant methods.
 * The default implementation of this interface is given in <code>Network.Representation.NetworkImplementation</code>
 * and the users are encouraged to subclass or delegate to this class.
 * @see Network.Representation.NetworkImplementation
 */
 
public interface Network {
    int getNoOfNodes();
    int getNoOfArcs();
    int getNoOfAlgorithms();
    void setNoOfAlgorithms(int setNo);
    NodeList getNodeList();
    NodeList getNodeListEnd();
    COM.objectspace.jgl.Array getAlgorithmList();
    void addNode(Node addedNode);
    void removeNode(Node deletedNode, NodeFactory nodeFactory, ArcFactory arcFactory);
    void addArc(Arc addedArc,Node fromNode, Node toNode);
    void removeArc(Arc deletedArc, ArcFactory arcFactory);
    NodeList getNextNodeInList(NodeList currentPosition);
    NodeList getPreviousNodeInList(NodeList currentPosition);
    void clean(NodeFactory nodeFactory, ArcFactory arcFactory); //Cleans Nodes and their Arcs, and returns them to any non-null factories
}

