package Network.Representation;

import Network.Representation.*;

import java.util.*;

/**
 * This interface provides a general template for nodes in a network. Users are encouraged to
 * use the default implementation of this interfave (<code>NodeImplementation</code>) by either subclassing
 * or delegating.
 * @see Network.Representation.NodeImplementation
 */
public interface Node {
    
    void initializeNode(int nodeSupply);
    void addOutBoundArc(Arc arc);
    void removeOutBoundArc(Arc arc);
    void addInBoundArc(Arc arc);
    void removeInBoundArc(Arc arc);
    int getNoOfInBoundArcs();
    int getNoOfOutBoundArcs();
    Network getContainerNetwork();
    double getSupply();
    void setSupply(double mySupply);
    ArcList getInBoundArcs();
    void setNoOfInBoundArcs(int setNumber);
    void cleanInBoundArcs(); //Arcs not collected; see cleanOutBoundArcs
    ArcList getOutBoundArcs();
    void setNoOfOutBoundArcs(int setNumber);
    void cleanOutBoundArcs(ArcFactory arcFactory); //Arcs not collected if arcFactory == null
    Vector getNodeAlgorithmInfo();
    void cleanNodeAlgorithmInfo();
    void setContainerNetwork(Network setNetwork);
    
}
